<?php
require_once('functions.php');
require_once('config.php');

/**
 * 署名検証
 */
function _verifySignature($consumerSecret){

	$oauthParams = _getOAuthParams();

	$oauthSignature = urldecode($oauthParams['oauth_signature']);

	unset($oauthParams['oauth_signature']); // 必要ないので削除
	unset($oauthParams['realm']);           // 必要ないので削除

	$oauthParams = array_merge($oauthParams, $_GET); // getパラメータを merge
	ksort($oauthParams); // param name で sort

	$baseParams = array(
		$_SERVER['REQUEST_METHOD'], // method
		'http://'.$_SERVER['SERVER_NAME'].$_SERVER['SCRIPT_NAME'], // request url
		http_build_query($oauthParams),
	);
	$baseParams = array_map('urlencode', $baseParams); // それぞれ urlencode
	$baseStr = implode('&', $baseParams);

	$key = $consumerSecret.'&';
	$signature = hash_hmac('sha1', $baseStr, $key, true);
	$signature = base64_encode($signature);
	$ret = ($signature == $oauthSignature);
	// error_log($oauthSignature.' '.($ret?'':'!').'= '.$signature."\n"); // 必要ならログ出力

	return $ret;
}

/**
 * authorization params を取得
 */
function _getOAuthParams(){

	$httpAuthorization = $_SERVER['HTTP_AUTHORIZATION'];
	$httpAuthorization = substr($httpAuthorization, strlen("OAuth ")); // 先頭の 'OAuth' を削除
	$tmpAry = explode(', ', $httpAuthorization);

	$oauthParams = array();
	foreach($tmpAry as $param){
		preg_match('/^(.+)="(.*)"$/', $param, $m);
		$oauthParams[$m[1]] = $m[2];
	}
	return $oauthParams;
}

$verify = _verifySignature(RESTFUL_CONSUMER_SECRET);

header('HTTP/1.1 200 OK');
header('Content-type: text/plain');
echo ($verify ? 'OK' : 'NG');
exit;
